/**
 *  Script Name  : dbInsertValueInterval.groovy
 *
 *  Purpose      : To demonstrate the usage of Scripting in Groovy. This is a script to insert
 *                    the values from all the files whose names start with 'outputTest.txt_' 
 *                    from the current directory, into the 'GroovyScriptTest' database table.
 *                    After processing each file, the script will move it into the 'outputFiles'
 *                    directory.
 *            
 *                   This script will work like a deamon thread which will continue to look for the similar 
 *                    files at regular intervals forever with a few minutes of sleep in between,
 *                    until the script is terminated by pressing Ctrl+C (interruption)
 *
 *  Environment  : Tested with -> Groovy Version: 2.4.3 JVM: 1.7.0_80 Vendor: Oracle Corporation OS: Windows 7
 *
 *  URL to refer : https://examples.javacodegeeks.com/jvm-languages/groovy
 *
 *  Author       : Raghavan alias Saravanan Muthu (Java Code Geeks)
 *  Date         : 11 Apr 2016, Monday
 */

package com.javacodegeeks.example.groovy.scripting;

// These two lines are for facilitating the System Classloader to pickup the 
// MySQL JDBC Connector Jar file from its classpath.  
//   1. Required when you invoke the script in GroovyConsole
//   2. Not required if you invoke the script in Command Prompt with a -cp option 
//        specifying the classpath to locate the jar file
@GrabConfig(systemClassLoader=true)
@Grab('mysql:mysql-connector-java:5.1.29')

import groovy.sql.Sql
import groovy.io.FileType

def url = DBProps.dbUrl
def user = DBProps.dbUser
def password = DBProps.dbPwd
def driver = DBProps.dbDriver
def tableName = DBProps.dbTblName

def baseDir = "."
def targetDir="./outputFiles"
def fileNamePattern = 'outputTest.txt_'

def dir = new File(baseDir)
def list, fileName, fileNameMatches
def orderlist = []
// interval to let the script sleep (1000ms = 1s, here it indicates 1 minute)
def SLEEP_INTERVAL = 1000*60*1
def MAX_ITERATIONS = 2, i = 1

// an utility method to print the current date and time in a specified format
def getNow()
{
    new Date().format("yyyyMMdd_HHmmss")
}

println " "
println "Script will try ${MAX_ITERATIONS} times for the matching input files while sleeping for ${SLEEP_INTERVAL/60000} minutes"

while(i++ <= MAX_ITERATIONS)
{
      println " "
      println "-------------------------------------------------------------------------------------------------"
      println getNow () + " =========== SCANNING for files matching with ${fileNamePattern} =================== "
      println "-------------------------------------------------------------------------------------------------"
      println " "
    
      dir = new File(baseDir)
    
    /* This will properly filter the files in the present directory 
     *  and allows ONLY the files and NOT any directories (subdirectories)
     */
    dir.eachFile(FileType.FILES) { file ->
       
       fileName = file.name
       fileNameMatches = false
       
       if(fileName.startsWith(fileNamePattern)) 
       {
           println "Processing the file -> $fileName"
           fileNameMatches = true
       }
    
       if(fileNameMatches)
       {
            // store all the lines from a file into a List, after trimming the spaces on each line
            list = new File(baseDir, fileName).readLines()*.trim()
            orderList = []
        
            // Process only the valid (non-empty) lines and prepare an Order instance
            // by calling the initOrder() method by passing the pipe delimited line of text
            list.each { it ->
                if(it.trim().length()>0)
                  orderList.add(new Order().initOrder(it))
            }    
    
            Sql.withInstance(url, user, password, driver) { sql ->
              orderList.each { it -> 
                  sql.execute """
                      INSERT INTO ${Sql.expand(tableName)}
                    (NAME, COUNTRY, PRICE, CURRENCY, PAYMENT_MODE)
                    VALUES
                    (
                        ${it.customerName}, ${it.country},
                        ${it.price}, ${it.currency}, ${it.paymentMode}
                    );
                  """
              }
            
              println "Total Rows Inserted to db : " + orderList.size()
            }
        
            // Move the processed file into a different directory
            // so that the same file will NOT be processed during next run
            "mv ${fileName} ${targetDir}".execute()
            println "File ${fileName} moved to dir [${targetDir}] successfully."
           }
        }
    
    println ""
    println getNow() + " ... script will sleep for ${SLEEP_INTERVAL} milliseconds..."
    println ""
    sleep(SLEEP_INTERVAL)
    println getNow() + " ......... script awaken ........ "
    println ""
}

println " "
println "----------------- "
println " == COMPLETED ==  "
println "----------------- "